package com.intersys.sample.jms;

import java.io.*;
import java.lang.Exception;
import javax.jms.Connection;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.Message;
import javax.jms.TextMessage;


/**
 * @author M.Mundt, InterSystems, March 2017
 *
 * Base class for JMS adapters. This should be subclassed by a class specific to the JMS server (ActiveMQ, etc.) 
 * with method overrides for InitConnection() and other methods that require non-standard calls to the vendor API
 * 
 *
 *****************************************************
 *                 ** N O T I C E **                 *
 *               - TEST/DEMO SOFTWARE -              *
 * This and related items are not supported by       *
 * InterSystems as part of any released product.     *
 * It is supplied by InterSystems as a demo/test     *
 * tool for a specific product and version.          *
 * The user or customer is fully responsible for     *
 * the maintenance of this software after delivery,  *
 * and InterSystems shall bear no responsibility nor *
 * liabilities for errors or misuse of this item.    *
 *                                                   *
 ***************************************************** 
 *
 *
 *	From docs for EnsLib.JavaGateway.InboundAdapter: 
 *	A JMS wrapper class should implement the following methods:
 *	 		.Connect(pJGClientConnectSpec,pTimeout,pInbound)
 *	 		.Disconnect(pInbound)
 *	 		.IsConnected(pInbound)
 *	 		.getErrorText() [returns empty string if no error on previous method call]
 *	 		.ReceiveMessage(pTimeout) [inbound only; pMsgIn return needs not be populated if timed out]
 *	 		.SendResponse(pMsgOut) [inbound only, optional-needed if OnProcessInput will return an object]
 *	 		.SendMessage(pMsgOut,pGetReply,pResponseTimeout) [outbound only; pMsgIn return needs not be populated if no responses are anticipated]
 *
 */

public class Wrapper {

	protected String ConnectionString;
	protected String ErrorText = null;
	protected Connection JMSConnection;
	protected Session JMSSession;
	protected Queue JMSQueue;
	protected MessageProducer JMSMessageProducer;
	protected MessageConsumer JMSMessageConsumer;
	protected BufferedWriter LogWriter;
	protected FileOutputStream OutStream;

	
	public Wrapper() {
		try {
			// This should be changed to allow the user to specify the log path as an Ensemble setting
			OutStream = new FileOutputStream("JMSWrapper_" + new java.util.Date().toString().replace(":", ".") + new java.util.Random().nextInt(1000000) + ".txt");
			LogWriter = new BufferedWriter(new OutputStreamWriter(OutStream, "utf-8"));
			LogWriter.write("Starting\n");
			LogWriter.flush();
		} catch (Exception e) {
			System.out.println("oops");
		}
	}
	
	protected void LogLine(String logMsg) {
		try {
			LogWriter.write(new java.util.Date() + "	" + logMsg+"\n");
			//LogWriter.flush();
		} catch (Exception e) {
			System.out.println("oops");
		}		
	}
	
	
	public Response ReceiveMessage(Integer timeout) throws Exception {	
		Response response=new Response();

		try {
			initConsumer();
			Message message = JMSMessageConsumer.receive(timeout);
			
			if (message != null) {
				javax.jms.TextMessage textMessage = (javax.jms.TextMessage) message;
				response.MessageText = textMessage.getText();
				response.IsMessage=true;
				message.acknowledge();
			}
		} catch (java.lang.Exception ex) {
			LogLine("Caught exception in ReceiveMessage. Clear Connection");
			LogLine(ex.toString());
			Disconnect(true);

			// Let Ensemble know there was an unhandled exception. Clean-up the old Connection/Queue/consumer
			response.IsException=true;
			response.IsMessage=false;
			response.ExceptionText=ex.toString();
			ErrorText=ex.toString();
		}
		
		return response;
	}
	
	
	public void SendMessage(String MessageContent, Boolean GetReply, Integer ResponseTimeout) throws Exception {
		// Note: Replys are not implemented. GetReply and ResponseTimeout will be ignored
		try {
			initProducer();
			TextMessage textMessage = JMSSession.createTextMessage(MessageContent);
			JMSMessageProducer.send(textMessage);
			//LogLine("Sent message["+MessageContent.length()+"]");
			LogLine("Sent message");
		} catch (java.lang.Exception ex) {
			ErrorText=ex.toString();
			LogLine("Caught exception in SendMessage. Clear Connection");
			LogLine(ex.toString());
			Disconnect(false);
			throw ex;
		}
	}
	
	
	public void Disconnect(Boolean IsInbound) {
		try {
			LogLine("Disconnect("+IsInbound+")");
			JMSSession.close();
			JMSConnection.stop();
			JMSConnection.close();
			LogWriter.flush();
			LogWriter.close();
		} catch (java.lang.Exception ex) {
			// We're cleaning up, so ignore exception thrown by stop()
		}

		JMSConnection=null;
		JMSSession=null;
		JMSQueue=null;
		JMSMessageConsumer=null;
		JMSMessageProducer=null;
	}
	
	
	public String getErrorText() {
		return ErrorText;
	}
	
	
	public void Connect(String ConnectionSpec, Integer Timeout, Boolean IsInbound) {
		LogLine("Connect()");

		ConnectionString = ConnectionSpec;
		
		try {
			if (IsInbound) {
				initConsumer();
			} else {
				initProducer();
			}
		} catch (java.lang.Exception ex) {
			ErrorText=ex.toString();
			LogLine("Caught exception in Connect");
			LogLine(ex.toString());
		}
	}
	
	
	protected void initConnection() throws Exception {
		// Implement in subclass
	}
	
	
	protected void initProducer() throws Exception {
		LogLine("initProducer()");
		if (this.JMSMessageProducer==null) {	
			LogLine("MessageProducer is null. Creating a new one.");

			initConnection();

			JMSMessageProducer = JMSSession.createProducer(JMSQueue);
		}
	} 

	
	protected void initConsumer() throws Exception {
		LogLine("initConsumer()");
		if (JMSMessageConsumer==null) {	
			LogLine("JMSMessageConsumer is null. Creating a new one.");			

			initConnection();

			JMSMessageConsumer = JMSSession.createConsumer(JMSQueue);
			JMSConnection.start();
		}
	} 

}


	
	